package batch

import (
	"context"
	"encoding/json"
	"io"
	"time"

	"github.com/aws/aws-sdk-go-v2/config"
	"github.com/aws/aws-sdk-go-v2/service/batch"
	"github.com/aws/aws-sdk-go-v2/service/batch/types"
	"github.com/go-resty/resty/v2"
	"github.com/google/uuid"
	"gitlab.bob.co.za/bob-public-utils/bobgroup-go-utils/logs"
	"gitlab.bob.co.za/bob-public-utils/bobgroup-go-utils/s3"
	"gitlab.bob.co.za/bob-public-utils/bobgroup-go-utils/utils"
)

const (
	JobDefinition                             = "batch-job-definition" // The job definition name on aws
	binaryPath                                = "/root/home/workers"   // The worker binary path in the docker container
	BatchJobMessageTypeS3 BatchJobMessageType = "s3"
)

// Queue names are the same as the batch Job queue names in aws
const (
	BatchJobQueueLow    BatchJobQueue = "batch-job-queue-low"
	BatchJobQueueMedium BatchJobQueue = "batch-job-queue-medium"
	BatchJobQueueHigh   BatchJobQueue = "batch-job-queue-high"
)

type BatchJobQueue string
type BatchJobMessageType string

type BatchJob struct {
	Name               *string
	Job                any
	FullJobDefinition  string
	FullJobQueue       string
	MessagesBucketName string
	IsDebug            bool
}

func SubmitJob(batchJob BatchJob) error {
	if batchJob.IsDebug {
		go func() {
			resty.New().R().
				SetBody(batchJob.Job).
				Post("http://127.0.0.1:3000/batch/")
		}()
		time.Sleep(time.Second * 1)
		return nil
	}

	cfg, err := config.LoadDefaultConfig(context.TODO(), config.WithRegion("af-south-1"))
	if err != nil {
		return err
	}

	batchClient := batch.NewFromConfig(cfg)

	if batchJob.Name == nil {
		id := uuid.New()
		jobID := "job" + id.String()
		batchJob.Name = utils.ValueToPointer(jobID)
	}

	err = uploadMessageToS3(batchJob)
	if err != nil {
		return err
	}

	command := []string{
		binaryPath,
		utils.PointerToValue(batchJob.Name),
	}

	environmentOverwrite := []types.KeyValuePair{{
		Name:  utils.ValueToPointer("BATCH_MESSAGE_TYPE"),
		Value: utils.ValueToPointer(string(BatchJobMessageTypeS3)),
	},
	}

	input := &batch.SubmitJobInput{
		JobDefinition: utils.ValueToPointer(batchJob.FullJobDefinition),
		JobName:       batchJob.Name,
		JobQueue:      utils.ValueToPointer(batchJob.FullJobQueue),
		ContainerOverrides: &types.ContainerOverrides{
			Command:     command,
			Environment: environmentOverwrite,
		},
	}
	_, err = batchClient.SubmitJob(context.TODO(), input)
	if err != nil {
		return err
	}
	return nil
}

func uploadMessageToS3(batchJob BatchJob) error {
	jobBytes, err := json.Marshal(batchJob.Job)
	if err != nil {
		return err
	}

	// Upload message
	_, err = s3.GetClient(batchJob.IsDebug).UploadWithSettingsRevised(jobBytes, batchJob.MessagesBucketName, s3.S3UploadSettings{
		FileName: utils.PointerToValue(batchJob.Name),
	})
	if err != nil {
		return err
	}

	return nil
}

func RetrieveMessageFromS3(filename string, messagesBucketName string, isDebug bool) ([]byte, error) {
	// get the file contents
	rawObject, err := s3.GetClient(isDebug).GetObject(messagesBucketName, filename, isDebug)
	if err != nil {
		return []byte{}, err
	}
	defer rawObject.Body.Close()

	// Read the file, unzip the data and unmarshall the json
	var bodyBytes []byte
	bodyBytes, err = io.ReadAll(rawObject.Body)
	if err != nil {
		logs.ErrorWithMsg(err, "Could not read file")
		return bodyBytes, err
	}
	return bodyBytes, nil
}
